#!/bin/sh
#
# Install bodhi's uBoot and debian rootfs on Kirkwood devices

# Copyright (c) 2016 Alberto Bursi <starshipeleven@outlook.com>
# Copyright (c) 2012 Jeff Doozan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# Version 2.0    [2016] Heavily revised, modularized and extended by Alberto Bursi to cover more devices
#                       and to run with its own toolbox
# Version 1.1    [1/24/2013] Revised by Daniel Richard G.
# Version 1.0    [7/14/2012] Initial Release by Jeff Doozan

##### MAINTAINER TIPS:
#
# -if you want to add support for a device, you need to add a new variable UBOOT_NEWNAME_URL with its link
#  in the "Definitions" section just below this paragraph, for its uboot download link.
#  The uboot md5 file link is generated dynamically inside the "install_uboot" function
#  by adding to UBOOT_MTD0_BASE_MD5 you find below the device-specific name and other things.
#  To have the new device show up in device selection phase and to add some more device-specific configuration,
#  scroll down into the "MAIN" section and add a new entry in the device selection case -- esac,
#  look at existing entries and copycat them.
#
# -if you want to change rootfs link, you need to change the link below in ROOTFS_URL, and the md5 link in ROOTFS_MD5
#
# -this script splits up code in functions for modularity reasons, so that you do not need to copy-paste the same code
#  over and over and over. There are 5 classes of functions used here:
#           --generic utility functions-- that deal with some specific action that is repeated
#            many times throughout the script
#           --functions that run a step of the installation routine--, calling also generic utility functions.
#           --device or vendor-specific functions-- that deal with device or vendor-specific actions
#            (usually killing stock firmware daemons that get in the way)
#           --install macrofunctions-- that mostly call a list of functions to run through all the install steps.
#           --disabled functions--, these are a safe way to keep dead code that might be useful in the future
#            inside this script, they contain remains from Jeff original script,
#            or things that do not work for other reasons.
#
#
# -any edit to this script MUST be done with a unix-aware text editor.
#  If you are on windows and you must edit it please use notepad++ that is unix-aware and will not screw up the encoding.
#
# -for editing it's recommended the use of an editor that has sintax highlighting for the very least
#  I personally recommend KWrite, Kate or Emacs because they have autocomplete and can shrink up functions.
#
###########


##############################
####
#### Definitions used throughout the script
####
##############################

####
#rootfs variables (debian-kirkwood-rootfs-bodhi-latest.tar.bz2 )
ROOTFS_URL="https://bitly.com/2agPH5E"
ROOTFS_MD5="https://raw.githubusercontent.com/mibodhi/kirkwood/kirkwood-2016-r1/checksum/debian-kirkwood-rootfs-bodhi-latest.tar.bz2.md5"

ROOTFS_TAR="debian-kirkwood-rootfs-bodhi-latest.tar"
ROOTFS="/tmp/rootfs/$ROOTFS_TAR.bz2"

####
#uboot-related variables

# uboot.bodhi.latest.environment
UBOOT_ENVIRONMENT_TXT_URL="https://bitly.com/2akfqZj"
#
# uboot.bodhi.latest.environment.img
UBOOT_ENVIRONMENT_IMG_URL="https://bitly.com/2akeSCD"
#
# uboot.bodhi.latest.environment.64K.img
UBOOT_ENVIRONMENT_IMG_64K_URL="https://bitly.com/2abh2oe"
#
# uboot.bodhi.latest.dockstar.mtd0.kwb
UBOOT_DOCKSTAR_URL="https://bitly.com/2aDBfsm"
#
# uboot.bodhi.latest.goflexhome.mtd0.kwb
UBOOT_GOFLEXHOME_URL="https://bitly.com/2aDBPGs"
#
# uboot.bodhi.latest.goflexnet.mtd0.kwb
UBOOT_GOFLEXNET_URL="https://bitly.com/2ayyzuS"
#
# uboot.bodhi.latest.iconnect.mtd0.kwb
UBOOT_ICONNECT_URL="https://bitly.com/2a6M5WP"
#
# uboot.bodhi.latest.netgear_ms2110.mtd0.kwb
UBOOT_NETGEAR_MS2110_URL="https://bitly.com/2al8b6E"
#
# uboot.bodhi.latest.nsa310.mtd0.kwb
UBOOT_NSA310_URL="https://bitly.com/2agOXxN"
#
# uboot.bodhi.latest.nsa310s.mtd0.kwb
UBOOT_NSA310S_URL="https://bitly.com/2a6LPax"
#
# uboot.bodhi.latest.nsa320.mtd0.kwb
UBOOT_NSA320_URL="https://bitly.com/2aDBT9w"
#
# uboot.bodhi.latest.nsa325.mtd0.kwb
UBOOT_NSA325_URL="https://bitly.com/2ayyyY5"
#
# uboot.bodhi.latest.pogo_e02.mtd0.kwb
UBOOT_POGO_E02_URL="https://bitly.com/2ayyCah"
#
# uboot.bodhi.latest.pogo_v4.mtd0.kwb
UBOOT_POGO_v4_URL="https://bitly.com/2aRl1YM"
#
# uboot.bodhi.latest.sheevaplug.mtd0.kwb
UBOOT_SHEEVAPLUG_URL="https://bitly.com/2aBZuFC"
#
# uboot.bodhi.latest.t5325.mtd0.kwb
UBOOT_T5325_URL="https://bitly.com/2aFz3h0"

#https://raw.githubusercontent.com/mibodhi/kirkwood/kirkwood-2016-r1/checksum/uboot.bodhi.latest.dockstar.mtd0.kwb.md5
# .$PLATFORM.mtd0.kwb.md5 will be appended to this
UBOOT_MTD0_BASE_MD5="https://raw.githubusercontent.com/mibodhi/kirkwood/kirkwood-2016-r1/checksum/uboot.bodhi.latest"

UBOOT_ENVIRONMENT_IMG_MD5="https://raw.githubusercontent.com/mibodhi/kirkwood/kirkwood-2016-r1/checksum/uboot.bodhi.latest.environment.img.md5"

UBOOT_MTD0="uboot.mtd0.kwb"
UBOOT_ENV="uboot.environment"


####
# Tools

# Download locations
MIRROR="https://dl.dropboxusercontent.com/u/47541136/linux/tools_for_Kirkwood_installer" # from Alberto Bursi's dropbox
# full link https://github.com/mibodhi/kirkwood/blob/kirkwood-2016-r1/tools/busybox?raw=true
#from github
MIRROR_HEADER="https://github.com/mibodhi/kirkwood/blob/kirkwood-2016-r1/tools"
MIRROR_FOOTER="?raw=true"

# Download links
BLPARAM_URL="$MIRROR/blparam"
NANDDUMP_URL="$MIRROR_HEADER/nanddump$MIRROR_FOOTER"
NANDWRITE_URL="$MIRROR_HEADER/nandwrite$MIRROR_FOOTER"
FLASH_ERASE_URL="$MIRROR_HEADER/flash_erase$MIRROR_FOOTER"
FW_PRINTENV_URL="$MIRROR_HEADER/fw_printenv$MIRROR_FOOTER"
FW_CONFIG_URL="$MIRROR_HEADER/fw_env.config$MIRROR_FOOTER"

BUSYBOX_URL="$MIRROR_HEADER/busybox$MIRROR_FOOTER"
MKE2FS_URL="$MIRROR/mke2fs"
E2FSCK_URL="$MIRROR_HEADER/e2fsck$MIRROR_FOOTER"
SGDISK_URL="$MIRROR/sgdisk"

WGET_WITH_SSL_URL="http://pogoplug.s3.amazonaws.com/wget"

# MD5 urls
GENERIC_MD5_URL=$MIRROR #from Alberto Bursi's dropbox as there are no md5 anywhere for tools.
BLPARAM_MD5="$GENERIC_MD5_URL/blparam.md5"
NANDDUMP_MD5="$GENERIC_MD5_URL/nanddump.md5"
NANDWRITE_MD5="$GENERIC_MD5_URL/nandwrite.md5"
FLASH_ERASE_MD5="$GENERIC_MD5_URL/flash_erase.md5"
FW_PRINTENV_MD5="$GENERIC_MD5_URL/fw_printenv.md5"
#FW_SETENV_MD5="$GENERIC_MD5_URL/fw_setenv.md5" this uses the same binary as fw_printenv, so no need to check it
FW_CONFIG_MD5="$GENERIC_MD5_URL/fw_env.config.md5"

BUSYBOX_MD5="$GENERIC_MD5_URL/busybox.md5"
MKE2FS_MD5="$GENERIC_MD5_URL/mke2fs.md5"
E2FSCK_MD5="$GENERIC_MD5_URL/e2fsck.md5"
SGDISK_MD5="$GENERIC_MD5_URL/sgdisk.md5"

#WGET_WITH_SSL_MD5="$GENERIC_MD5_URL/wget.md5" not used currently as we cannot download md5 for it

# Where we download and keep tools temporarily
TOOLS="/tmp/tools"

# Tools and their place on the tmpfs
BLPARAM="$TOOLS/blparam"
NANDDUMP="$TOOLS/nanddump"
NANDWRITE="$TOOLS/nandwrite"
FLASH_ERASE="$TOOLS/flash_erase"
FW_PRINTENV="$TOOLS/fw_printenv"
FW_SETENV="$TOOLS/fw_setenv"
FW_CONFIG="$TOOLS/etc/fw_env.config"

BUSYBOX="$TOOLS/busybox"
MKE2FS="$TOOLS/mke2fs"
E2FSCK="$TOOLS/e2fsck"
SGDISK="$TOOLS/sgdisk"

WGET_WITH_SSL="$TOOLS/wget"



####
# Recovery

#uimage name will be generated when flashing
#uImage-kirkwood-$RECOVERY_PLATFORM
#uImage md5 will be the above + ".md5"

RECOVERY_ROOTFS="lede-kirkwood-generic-squashfs.img"

#download links

MAIN_RECOVERY_URL="https://dl.dropboxusercontent.com/u/47541136/linux/kirkwood_recovery" # from Alberto Bursi's dropbox
MAIN_RECOVERY_MD5_URL="https://dl.dropboxusercontent.com/u/47541136/linux/kirkwood_recovery" # from Alberto Bursi's dropbox
RECOVERY_ROOTFS_URL="$MAIN_RECOVERY_URL/$RECOVERY_ROOTFS"
RECOVERY_ROOTFS_MD5_URL="$MAIN_RECOVERY_URL/$RECOVERY_ROOTFS.md5"

#####
#
#Device-specific definitions and commands are at the end of this script, in the MAIN, inside the "case" statement
#
#####


##############################
####
#### Some common utility functions used by the script's macrofunctions
####
##############################

verify_md5 (){
  file="$1"
  md5="$2"

  echo "verify file $file with md5 $md5"

  check_md5=$( cat "$md5" | cut -d' ' -f1)

  echo "$check_md5"

  file_md5=$(md5sum "$file" | cut -d' ' -f1)

  echo "$file_md5"

  if [ "$check_md5" = "$file_md5" ]; then
    echo "passed"
    return 0
  else
    echo "failed"
    return 1

  fi
}

download_and_verify (){ # add curl as a fallback     curl -O http://server1.cyberciti.biz/file.tar.gz

  file_dest="$1"
  file_url="$2"

  #now this comes from outside as we keep md5 on a different server
  md5_url="$3"

  md5_dest="$file_dest.md5"

  echo "$file_dest"
  echo "$file_url"
  echo "$md5_url"
  echo "$md5_dest"


  # Always download a fresh MD5, in case a newer version is available
  # the no-check-certificate in wget is needed because 99% of the cases the stock firmware will lack
  # a "legit certificates" list, so we must ask wget to not check the ssl certificate's validity.
  if [ -f "$md5_dest" ]; then rm -f "$md5_dest"; fi
  wget -O "$md5_dest" "$md5_url" --no-check-certificate
  # retry the download if it failed
  if [ ! -f "$md5_dest" ]; then
    wget -O  "$md5_dest" "$md5_url" --no-check-certificate
    if [ ! -f "$md5_dest" ]; then
      return 1 # Could not get md5
    fi
  fi

  # Download the file
  wget -O  "$file_dest" "$file_url" --no-check-certificate
  # retry the download if it failed
  verify_md5 "$file_dest" "$md5_dest"
  if [ "$?" != "0" ]; then
    echo "Download failed or MD5 did not match, try again"
    if [ -f "$file_dest" ]; then rm -f "$file_dest"; fi
    wget -O  "$file_dest" "$file_url" --no-check-certificate
    verify_md5 "$file_dest" "$md5_dest"
    if [ "$?" != "0" ]; then
      rm -f "$md5_dest"
      ERROR="## $file_dest verification failed 2 times"
      error_harmless
    fi
  fi

  rm -f "$md5_dest"
  return 0
}

install (){
  file_dest="$1"
  file_url="$2"
  file_md5_url="$3" # md5 file url
  file_pmask="$4"  # Permissions mask

    echo
    echo "# Installing $file_dest..."

    if [ "$file_md5_url" = "NO_CHECK" ]; then
      # Download the file without checking md5
      wget -O  "$file_dest" "$file_url" --no-check-certificate
    fi

    if [ "$file_md5_url" != "NO_CHECK" ]; then
      download_and_verify "$file_dest" "$file_url" "$file_md5_url"
      if [ "$?" != "0" ]; then
        ERROR="## Could not install $file_dest from $file_url."
        error_harmless
      fi
    fi

   echo " chmodding ""$file_pmask" " -- " "$file_dest"
    chmod "$file_pmask" "$file_dest"

    echo "# Successfully installed $file_dest."

  return 0
}

decide_what_drive_we_work_with(){

#since for various reasons I cannot get a RELIABLE way to find what device is friggin / I resort to ask the user

#looking at drives and detecting if sda sdb or mmcblk0 are there.
drive_sda=$(fdisk -l | grep /dev/sda | grep bytes )
drive_sdb=$(fdisk -l | grep /dev/sdb | grep bytes )
drive_mmcblk0=$(fdisk -l | grep /dev/mmcblk0 | grep bytes )
DISK=""

#deciding if there are no drives at all
if [ "$drive_sda" = "" ] && [ "$drive_mmcblk0" = "" ]; then
  ERROR="## no disks detected, exiting now"
  error_harmless
fi

#deciding if there is ONLY sda
if [ "$drive_sda" != "" ] && [ "$drive_sdb" = "" ]; then
  DISK="/dev/sda"
fi

#deciding if there is ONLY mmcblk0
if [ "$drive_mmcblk0" != "" ] && [ "$drive_sdb" = "" ]; then
  DISK="/dev/mmcblk0"
fi

#if after these checks DISK is still empty, we ask the user.
if [ "$DISK" = "" ]; then
  echo
  echo "what drive you want to install Debian in?"
  echo "(1,2,3,4,sd)"
  DISK=$(fdisk -l | grep /dev/sda | grep bytes |  awk '{if ($3 >= 1) print $1, $3, $4}')

  if [ "$DISK" != "" ]; then
    echo "1. $DISK"
  fi

  DISK=$(fdisk -l | grep /dev/sdb | grep bytes | awk '{if ($3 >= 1) print $1, $3, $4}' )
  if [ "$DISK" != "" ]; then
    echo "2. $DISK"
  fi

  DISK=$(fdisk -l | grep /dev/sdc | grep bytes | awk '{if ($3 >= 1) print $1, $3, $4}' )
  if [ "$DISK" != "" ]; then
    echo "3. $DISK"
  fi

  DISK=$(fdisk -l | grep /dev/sdd | grep bytes | awk '{if ($3 >= 1) print $1, $3, $4}' )
  if [ "$DISK" != "" ]; then
    echo "4. $DISK"
  fi

  DISK=$(fdisk -l | grep /dev/mmcblk0 | grep bytes | awk '{if ($3 >= 1) print $1, $3, $4}')
  if [ "$DISK" != "" ]; then
    echo "sd. $DISK"
  fi

read -r SELECTION

DISK=""

#making a cycle so any mistake does not terminate the script here
while [ "$DISK" = ""  ]; do
case $SELECTION in

    1)
      DISK="/dev/sda"
    ;;
    2)
      DISK="/dev/sdb"
    ;;
    3)
      DISK="/dev/sdc"
    ;;
    4)
      DISK="/dev/sdd"
    ;;
    sd)
      DISK="/dev/mmcblk0"
    ;;
    *)
      echo "not recognized, try again"
    ;;
esac

done

fi

echo
echo "installation drive is $DISK"
echo

PART_1="${DISK}1"
PART_2="${DISK}2"

echo
echo "---unmounting installation drive"
echo

umount -l "$PART_1"
umount -l "$PART_2"

}

run_fw_printenv_setenv_in_chroot(){
command="$1"
args="$2"

#Surprise, surprise!
#In my Zyxel nsa310 can bind mount anything anywhere, but NOT /dev
#so making a chroot is pointless as it won't see /dev/mtd0
#I resort to a simpler hack I already did in the past
#commenting out the chrooting that may be useful for the future
#theoretically I can use makedevs to create other device files in the new dev
#but my hack is simpler and I'm sure it will work

#preparing folder mounts for chroot

# mkdir -p "$TOOLS/proc"
# mount proc "$TOOLS/proc" -t proc
#
# mkdir -p "$TOOLS/sysfs"
# mount proc "$TOOLS/sysfs" -t sysfs
#
# mkdir -p "$TOOLS/dev"
# mount -o bind "/dev" "$TOOLS/dev"
#
# #chrooting to give the command only
#
# chroot "$TOOLS"/ ./"$command" "$args"
#
# #unmounting stuff
# umount "$TOOLS/proc"
# umount "$TOOLS/sysfs"
# umount "$TOOLS/dev"


#copying /etc to another folder
mkdir -p "$TOOLS/etc_original"
cp -rf "/etc/." "$TOOLS/etc_original"
#making a tmpfs over the damn /etc
mount -t tmpfs -o rw tmpfs "/etc"
#copying the /etc contents into it,
cp -rf "$TOOLS/etc_original/." "/etc/"
#changing the configs I need to change and then run the command.
cp -rf "$TOOLS/etc/fw_env.config" "/etc/"

$command $args

#cool, we now remove the tmpfs
umount "/etc"

#and dump the clone of stock /etc
rm -r "$TOOLS/etc_original"

}

read_env_stock_uboot(){

#reads a stock env and returns its value, using various ways

ENV_NAME=$1
ERROR_DISABLE=$2

#cleaning the variable
VALUE=""

#checking if stock blparam (afaik present only in first-gen kirkwood devices, and always in this place)
#returns 0, i.e. if the env exists at all, if it returns 0 we use it.
#also keeping this darn monster silent as its output is scary
if [ -f "/usr/local/cloudengines/bin/blparam" ]; then
  #if [  $(/usr/local/cloudengines/bin/blparam >/dev/null 2>&1) ] ; then
    echo
    echo "stock blparam detected, reading envs with it"
    VALUE="$(/usr/local/cloudengines/bin/blparam 2>/dev/null  | grep "^$ENV_NAME=" | sed "s@$ENV_NAME=@@g"  )"
  #fi
fi

# if the value is loaded, we exit the function
if [ "$VALUE" != "" ] ; then
  echo
  echo "env read, it is $VALUE"
  return 0
fi

#repeating the same trick as above but with our own blparam
#if [  $(blparam >/dev/null 2>&1) ] ; then
#  echo
#  echo "our blparam detected, reading envs with it"
#  VALUE="$(blparam 2>/dev/null  | grep "^$ENV_NAME=" | sed "s@$ENV_NAME=@@g"  )"
#fi

#for some reason it cannot check if our blparam gives a proper error code
#resorting to simply calling it and filtering all entries without a "="
#this will filter all garbage it prints that isn't the envs dump
VALUE="$(blparam 2>/dev/null  | grep "^$ENV_NAME=" | sed "s@$ENV_NAME=@@g"  )"

# if the value is loaded, we exit the function
if [ "$VALUE" != "" ] ; then
  echo
  echo "env read, it is $VALUE"
  return 0
fi

#repeating the same trick as above but with stock fw_printenv
#checking that there is a fw_printenv at all or it would screw up error code and the if
export PATH="$ORIGINAL_PATH"
#if [  $("$TOOLS/which" fw_printenv) ] ; then
#  if [ ! $(fw_printenv "$ENV_NAME" >/dev/null 2>&1) ] ; then
#    echo
#    echo "stock fw_printenv reports an env, reading it"
    VALUE="$(fw_printenv "$ENV_NAME"  2>/dev/null )"
#  fi
#fi
#getting PATH back to use our own toolbox
export PATH="$TOOLS"
#finishing the env filtering with our own toolbox
VALUE="$(echo "$VALUE"  | sed "s@$ENV_NAME=@@g" )"

# if the value is loaded, we exit the function
if [ "$VALUE" != "" ] ; then
  echo
  echo "env read, it is $VALUE"
  return 0
fi

#repeating the same trick as above but with our own fw_printenv and the stock config
#if [ ! $(fw_printenv "$ENV_NAME" >/dev/null 2>&1) ] ; then
#  echo
#  echo "our fw_printenv with stock config reports an env, reading it"
  VALUE="$(fw_printenv "$ENV_NAME" 2>/dev/null  | sed "s@$ENV_NAME=@@g"  )"
#fi

# if the value is loaded, we exit the function
if [ "$VALUE" != "" ] ; then
  echo
  echo "env read, it is $VALUE"
  return 0
fi

if [ "$VALUE" = "" ]; then
  #if the error isn't disabled we trigger the error
  if [ "$ERROR_DISABLE" != "no-error"  ] ; then
    ERROR="## Could not read $ENV_NAME uboot variable."
    error_harmless
  fi
fi

#exporting our own tools path so we use tools from our toolbox again, just in case
export PATH="$TOOLS"

return 0

}

read_all_env_stock_uboot(){

#reads all stock envs and returns that, using various ways

#cleaning the variable
VALUE=""

#checking if stock blparam is there (afaik present only in first-gen kirkwood devices, and always in this place)
#and if it returns 0, i.e. if the envs exist at all, if it returns 0 we use it.
#also keeping this darn monster silent as its output is scary
if [ -f "/usr/local/cloudengines/bin/blparam" ] ; then
#   if [ ! $(/usr/local/cloudengines/bin/blparam >/dev/null 2>&1) ] ; then
    echo
    echo "stock blparam detected, reading envs with it"
    VALUE="$(/usr/local/cloudengines/bin/blparam 2>/dev/null | grep "=" )"
#  fi
fi

# if the value is loaded, we exit the function
if [ "$VALUE" != "" ] ; then
  echo
  printf "envs read, they are %s" "$VALUE"
  return 0
fi

#repeating the same trick as above but with our own blparam
# if [ ! $(blparam >/dev/null 2>&1) ] ; then
#   echo "our blparam detected envs, reading envs with it"
#   VALUE=$(blparam 2>/dev/null | grep "=" )
# fi

#for some reason it cannot check if our blparam gives a proper error code
#resorting to simply calling it and filtering all entries without a "="
#this will filter all garbage it prints that isn't the envs dump
VALUE=$(blparam 2>/dev/null | grep "=" )

# if the value is loaded, we exit the function
if [ "$VALUE" != "" ] ; then
  echo
  printf "envs read, they are %s" "$VALUE"
  return 0
fi

#repeating the same trick as above but with stock fw_printenv
#checking that there is a fw_printenv at all or it would screw up error code and the if
export PATH="$ORIGINAL_PATH"
# if [ $("$TOOLS/which" fw_printenv) ] ; then
#   if [ ! $(fw_printenv >/dev/null 2>&1) ]  ; then
#     echo "stock fw_printenv detected, reading envs with it"
    VALUE=$(fw_printenv  2>/dev/null )
#   fi
# fi
export PATH="$TOOLS"

# if the value is loaded, we exit the function
if [ "$VALUE" != "" ] ; then
  echo
  printf "envs read, they are %s" "$VALUE"
  return 0
fi

#repeating the same trick as above but with our own fw_printenv and the stock config
# if [ ! $(fw_printenv >/dev/null 2>&1) ] ; then
  echo "our fw_printenv with stock config reports an env, reading it"
  VALUE=$(fw_printenv 2>/dev/null  )
# fi

# if the value is loaded, we exit the function
if [ "$VALUE" != "" ] ; then
  echo
  printf "envs read, they are %s" "$VALUE"
  return 0
fi

if [ "$VALUE" = "" ]; then
  ERROR="## Could not read stock uboot envs."
  error_harmless
fi

#exporting our own tools path so we use tools from our toolbox again, just in case
export PATH="$TOOLS"

return 0

}

read_env_modern_uboot(){

#reads a modern uboot env and returns its value, using various ways

ENV_NAME=$1

#cleaning the variable
VALUE=""

#if ENV_NAME is empty we don't filter output as we need a full dump
echo "we now try reading the envs from an older version of our own uboot envs"

VALUE=$(run_fw_printenv_setenv_in_chroot "fw_printenv $ENV_NAME")

if [ "$?" != "0" ]; then
  printf "%s" "$VALUE"
  echo "the above command failed, we return an error code"
  return 1
fi

printf "%s" "$VALUE"
return 0
}

error_harmless(){

echo
echo "##"
echo "##"
echo "## INSTALLATION FAILED!"
echo "##"
echo "## $ERROR"
echo "##"
echo "##"
echo "## THE INSTALLATION FAILED BUT NO"
echo "## DAMAGE WAS MADE, YOU CAN SAFELY"
echo "## SHUTDOWN YOUR DEVICE"
echo "##"
echo "## POST THE FULL LOG OF THIS INSTALLATION"
echo "## ON FORUMS FOR FURTHER ASSISTENCE"
echo "##"
echo "## $SECOND_MESSAGE"

exit 1

}

error_dont_reboot(){

echo
echo "##"
echo "##"
echo "## INSTALLATION FAILED!"
echo "##"
echo "## $ERROR"
echo "##"
echo "##"
echo "## YOUR DEVICE MAY BE IN AN UNUSABLE STATE."
echo "## DO NOT REBOOT OR POWER OFF YOUR DEVICE"
echo "##"
echo "## POST THE FULL LOG OF THIS INSTALLATION"
echo "## ON FORUMS FOR FURTHER ASSISTENCE"
echo "##"
echo "## $SECOND_MESSAGE"

exit 1

}

end_installation_message_reboot(){

echo
echo "You can now reboot your device into Debian."
echo "If your device does not start Debian after rebooting,"
echo "you may need to restart the device by disconnecting the power."
echo ""
echo "The new root password is 'root'  Please change it immediately after"
echo "logging in."
echo ""
echo "Reboot now? [Y/n] "
read -r IN
if [ "$IN" = "" ] || [ "$IN" = "y" ] || [ "$IN" = "Y" ]
then
  /sbin/reboot
fi


}

flash_nand_image(){

# Install a disk image on a nand partition

echo ""
echo "# Installing $WHAT_WE_FLASH"

rm -f "$TOOLS/$WHAT_WE_FLASH"

download_and_verify "$TOOLS/$WHAT_WE_FLASH" "$IMG_URL" "$IMG_MD5_URL"
if [ "$?" != "0" ]; then
  ERROR="## Could not download or verify new $WHAT_WE_FLASH download."
  error_harmless
fi

IMG_SIZE="GET_SIZE"
#now that we have the file we can check its size, convert in hex and fill this variable
#lede firmares are already padded usually so there should be no issue
if [ "$IMG_SIZE" = "GET_SIZE" ] ; then
  SIZE_IN_HEX="$(printf "%x\n" "$(wc -c "$TOOLS/$WHAT_WE_FLASH" | awk '{print $1}' )")"
  IMG_SIZE="-l 0x$SIZE_IN_HEX"
fi

#downloading md5 to use for verification BEFORE writing things to NAND
#so if this fails we can still call an error_harmless and everything will be fine
rm -f "$TOOLS/$WHAT_WE_FLASH.md5"
wget -O "$TOOLS/$WHAT_WE_FLASH.md5" "$IMG_MD5_URL" --no-check-certificate
  if [ ! -f "$TOOLS/$WHAT_WE_FLASH.md5" ]; then
    wget -O  "$TOOLS/$WHAT_WE_FLASH.md5" "$IMG_MD5_URL" --no-check-certificate
    if [ ! -f "$TOOLS/$WHAT_WE_FLASH.md5" ]; then
      ERROR="## Could not download the md5 to verify the $WHAT_WE_FLASH."
      error_harmless
    fi
  fi


flash_erase $FLASH_ERASE_ARGS
#need to add a check on the output here.

nandwrite $NAND_WRITE_ARGS
#need to add a check on the output here.

#must delete the flashed image from the tmpfs as some devices (like zyxels) like to waste ram for their rootfs
rm -f "$TOOLS/$WHAT_WE_FLASH"

# dump mtd and compare the checksum, to make sure it installed properly
nanddump --noecc --omitoob -l $IMG_SIZE  $NANDDUMP_ARGS

echo "## Verifying new $WHAT_WE_FLASH..."

#verification to see if flash went well
verify_md5 "$TOOLS/$WHAT_WE_FLASH.dump" "$TOOLS/$WHAT_WE_FLASH.md5"
if [ "$?" != "0" ]; then
  rm -f "$TOOLS/$WHAT_WE_FLASH.md5"

  ERROR="## $WHAT_WE_FLASH was not properly installed to NAND."

      ##MAKE GENERIC BACKUP FUNCTION FOR THIS
      #SECOND_MESSAGE="## Make a backup of the content of $BACKUP_DIR in the installation drive someplace safe"

  "$ERROR_CALL"
else
  echo "# $WHAT_WE_FLASH verified successfully!"
fi

rm -f "$TOOLS/$WHAT_WE_FLASH.md5"

}

##############################
####
#### functions doing a step in the install process
####
##############################

#this still needs to be hooked into error system
backing_up_uboot_and_envs(){

#backing up stock envs and stock uboot

#if the disk to work with isn't defined, we run the function to decide what drive we work with
if [ "$DISK" = "" ]; then
  decide_what_drive_we_work_with
fi

#mounting rootfs partition
mkdir -p "/tmp/rootfs"

echo
echo "partition to mount is $PART_1"

mount "$PART_1" "/tmp/rootfs"

BACKUP_DIR="/tmp/rootfs/uboot_backups"
mkdir -p "$BACKUP_DIR"

#name of uboot backup
UBOOT_ORIGINAL="uboot-$PLATFORM-mtd0.kwb"

#assigning MTD0 at DEV_MTD0
DEV_MTD0="$STOCK_MTD0"

#defining current date
DATE="$(date +%F_%H-%M)"

#dumping all envs from stock or modern uboot
read_all_env_stock_uboot
if [ "$VALUE" != "" ]; then
  echo "read_env_stock_uboot ran correctly"
  printf "%s" "$VALUE" > "$BACKUP_DIR/stock_env_backup-$DATE.txt"

  echo
  echo "reading stock_env_backup.txt"
  echo
  cat "$BACKUP_DIR/stock_env_backup-$DATE.txt"

fi

read_env_modern_uboot
if [ "$VALUE" != "" ]; then
  echo "read_env_modern_uboot ran correctly"
  printf "%s" "$VALUE" > "$BACKUP_DIR/modern_env_backup-$DATE.txt"

  echo
  echo "reading modern_env_backup.txt"
  echo
  cat "$BACKUP_DIR/modern_env_backup-$DATE.txt"
fi

# dump mtd0
nanddump --noecc --omitoob -l 0x80000 -f "$BACKUP_DIR/$UBOOT_ORIGINAL-$DATE" "$DEV_MTD0"

umount "$PART_1"

}

create_tmpfs(){
#we generate our own tmpfs, then we mount it as our own tool directory.
mkdir -p $TOOLS

#unmounting any previous mounting of this
umount -l $TOOLS

mount -t tmpfs -o rw,dev,exec,suid tmpfs $TOOLS

if [ "$?" != "0" ]; then
  ERROR="## Could not make the tmpfs to install tools in."
  error_harmless
fi

}

check_bad_blocks(){
# checking if there are bad blocks

# Example of bad block in mtd0:
#
# dmesg | grep -i 'bad'
#
# [    2.413231] Scanning device for bad blocks
# [    2.417731] Bad eraseblock 3 at 0x000000060000
#
#
# This bad block is the 4th block which resides within mtd0 (block 0 to block 7). So u-boot should not be flashed.

#we do a loop to check that no block in the first MB or so is good, we don't care about the rest.

#128k = 131072 bytes -> 0x20000 in hexadecimal and we sum 0x20000 to it to get our addresses.

for f in 00000 20000 40000 60000 80000 a0000 c0000 e0000 ; do

# call dmesg, show only lines with "bad eraseblock" lower or uppercase,
#remove "0x" and any trailing zeroes so we get only the actual number of the hex address
#then finally check if in these lines there is the address we are looking for (both lower or upper and single word
#so we don't match also 200000 when looking for 20000)
dmesg | grep -i 'bad eraseblock' | sed 's/0x[0-0][0-0]*//' | grep -iw $f

if [ "$?" = "0" ]; then
    ERROR="## Bad blocks in 0x$f area on NAND, uboot must be installed manually."
    error_harmless
fi

echo "no bad blocks on 0x$f"

done

}

download_tools(){

#creating the tmpfs to download tools into
create_tmpfs

# Create the tools and TOOLS/etc directory if it doesn't already exist
mkdir -p "$TOOLS/etc"

#downloading manually a wget that can download from https links
wget -O  "$WGET_WITH_SSL"           "$WGET_WITH_SSL_URL"
if [ "$?" != "0" ]; then
  ERROR="## Could not download wget."
  error_harmless
fi
chmod "755" "$WGET_WITH_SSL"

#downloading manually busybox with the new wget, then manually checking
"$WGET_WITH_SSL" -O  "$BUSYBOX"       "$BUSYBOX_URL" --no-check-certificate
"$WGET_WITH_SSL" -O  "$BUSYBOX.md5"       "$BUSYBOX_MD5" --no-check-certificate
verify_md5 "$BUSYBOX" "$BUSYBOX.md5"
  if [ "$?" != "0" ]; then
    echo "Download failed or MD5 did not match, try again"
    if [ -f "$BUSYBOX" ]; then rm -f "$BUSYBOX"; fi
    wget -O  "$BUSYBOX" "$BUSYBOX_URL" --no-check-certificate
    verify_md5 "$BUSYBOX" "$BUSYBOX.md5"
    if [ "$?" != "0" ]; then
      ERROR="## Could not verify busybox download."
      error_harmless
    fi
  fi

chmod "755" "$BUSYBOX"

#backing up the standard PATH so we can use it later too.
ORIGINAL_PATH="$PATH"


echo "new PATH is now $PATH"

#changing PATH so we call stuff from our own folders from now on
#placing it before the stock path so all commands will be first called from here
#first goes the tools directory so we should call the right wget
export PATH="$TOOLS"

#adding symlinks to all the tools

#moving into the folder.
cd $TOOLS
if [ "$?" != "0" ]; then
  ERROR="## Could not move to $TOOLS folder."
  error_harmless
fi

$BUSYBOX ln -s $BUSYBOX ln #first is busybox's own symlink making a symlink for itself, then we go on

for f in  [ [[ acpid addgroup adduser adjtimex ar arping ash awk basename blockdev brctl bunzip2 bzcat bzip2 cal cat chgrp chmod chown chroot chvt clear cmp cp cpio crond crontab cut date dc dd deallocvt delgroup deluser df dirname dmesg dnsdomainname dos2unix dpkg dpkg-deb du dumpkmap dumpleases echo ed egrep eject env expand expr false fbset fdflush fdisk fgrep find fold free freeramdisk fsck.minix ftpget ftpput getopt getty grep gunzip gzip halt head hexdump hostid hostname httpd hwclock id ifconfig ifdown ifup init ionice ip ipcalc kill killall klogd last length less linuxrc loadfont loadkmap logger login logname logread losetup ls lzcat makedevs md5sum mdev mesg microcom mkdir mkfifo mkfs.minix mknod mkswap mktemp more mount mt mv nameif nc netstat nslookup od openvt passwd patch pidof ping ping6 pivot_root poweroff printf ps pwd rdate readlink realpath reboot renice reset rm rmdir route rpm rpm2cpio run-parts sed setkeycodes sh sha1sum sha256sum sha512sum sleep sort start-stop-daemon strings stty su sulogin swapoff swapon sync syslogd tac tail tar tee telnet telnetd test tftp time timeout top touch tr traceroute traceroute6 true tty tunctl udhcpc udhcpd umount uname uncompress unexpand uniq unix2dos unlzma unzip uptime usleep uudecode uuencode vconfig vi vlock watch watchdog wc which who whoami xargs yes zcat ; do

ln -s $BUSYBOX "$f"
done

chmod -R "755" $TOOLS

ls -l

cd
if [ "$?" != "0" ]; then
  ERROR="## Could not move back to / folder."
  error_harmless
fi
}

download_uboot_tools(){
#keeping uboot tools separated so we can use as little tmpfs/ram space as possible

install "$BLPARAM"          "$BLPARAM_URL"      "$BLPARAM_MD5"         755
    if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify blparam download."
      error_harmless
    fi
install "$NANDWRITE"        "$NANDWRITE_URL"    "$NANDWRITE_MD5"       755
    if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify nandwrite download."
      error_harmless
    fi
install "$NANDDUMP"         "$NANDDUMP_URL"     "$NANDDUMP_MD5"        755
    if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify nanddump download."
      error_harmless
    fi
install "$FLASH_ERASE"      "$FLASH_ERASE_URL"  "$FLASH_ERASE_MD5"     755
    if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify flash_erase download."
      error_harmless
    fi
install "$FW_PRINTENV"      "$FW_PRINTENV_URL"  "$FW_PRINTENV_MD5"     755
    if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify fw_printenv download."
      error_harmless
    fi

if [ ! -f "$FW_SETENV" ]; then
  ln -s "$FW_PRINTENV" "$FW_SETENV" 2> /dev/null
fi

install "$FW_CONFIG"        "$FW_CONFIG_URL"    "$FW_CONFIG_MD5"       644
    if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify fw_config download."
      error_harmless
    fi

}

download_formatting_tools(){

#keeping format tools separated so we can use as little tmpfs/ram space as possible

#install "$SGDISK"           "$SGDISK_URL"       "$SGDISK_MD5"          755 disabled until I recompile a sgdisk that works
install "$E2FSCK"           "$E2FSCK_URL"       "$E2FSCK_MD5"          755
if [ "$?" != "0" ]; then
    ERROR="## Could not download or verify e2fsck download."
    error_harmless
fi

install "$MKE2FS"           "$MKE2FS_URL"       "$MKE2FS_MD5"          755
if [ "$?" != "0" ]; then
    ERROR="## Could not download or verify mke2fs download."
    error_harmless
fi

}

download_and_write_rootfs(){

#if the disk to work with isn't defined, we run the function to decide what drive we work with
if [ "$DISK" = "" ]; then
  decide_what_drive_we_work_with
fi

echo
echo "---unmounting installation drive"
echo

umount -l "$PART_1"
umount -l "$PART_2"


###
#  make dirs & mount
echo "---mounting partitions"
mkdir -p "/tmp/rootfs"

echo
echo "partition to mount is $PART_1"
sleep 5

mount "$PART_1" "/tmp/rootfs"
if [ "$?" != "0" ]; then
  ERROR="## Mounting of rootfs failed, installation cannot proceed."
  error_harmless
fi

echo "mounted partitions are"
mount
echo
echo

echo "---checking that the stuff was mounted and writable"
# Just because it's better to.
echo "testing if this is readable" > "/tmp/rootfs/testfile"
if [ ! -f "/tmp/rootfs/testfile" ]; then
  ERROR="## Rootfs partition not writable."
  error_harmless
fi

#removing the test file
rm "/tmp/rootfs/testfile"
echo "---everything writable"
echo

#downloading rootfs in the rootfs partition, aborting if fails the md5 check.

download_and_verify "$ROOTFS" "$ROOTFS_URL" "$ROOTFS_MD5"
if [ "$?" != "0" ]; then
  ERROR="## Could not install $ROOTFS from $ROOTFS_URL."
  error_harmless
fi

#  extract the rootfs
echo "---extracting debian rootfs to install Debian"

cd "/tmp/rootfs/"
if [ "$?" != "0" ]; then
  ERROR="## Could not move into /tmp/rootfs/"
  error_harmless
fi

bunzip2 "$ROOTFS"
if [ "$?" != "0" ]; then
  ERROR="## Could not bunzip2 $ROOTFS from $ROOTFS_URL."
  error_harmless
fi

tar xvf "$ROOTFS_TAR"
if [ "$?" != "0" ]; then
  ERROR="## Could not untar $ROOTFS from $ROOTFS_URL, exiting."
  error_harmless
fi

rm "$ROOTFS_TAR"
cd
if [ "$?" != "0" ]; then
  ERROR="## Could not move into / again"
  error_harmless
fi


#  always make sure you're sync'd!
sync

#checking to see if it went right
if [ ! -f "/tmp/rootfs/etc/fstab" ]; then
    ERROR="root files not extracted correctly! Aborting!"
    error_harmless
fi
echo "---everything fine"

if [ -f "/sbin/e2label" ]; then # this means we are inside a Debian rootfs
  umount "$PART_1"
  /sbin/e2label "$PART_1" "rootfs"
    if [ "$?" != "0" ]; then
      ERROR="## Could not label the rootfs"
      error_harmless
    fi
fi

umount -l "$PART_1"
umount -l "$PART_2"

}

flash_uboot(){

# Install uBoot on /dev/mtd0

#setting primary variables for child function
WHAT_WE_FLASH="uboot"
MTD_PART="$STOCK_MTD0"
IMG_URL="$UBOOT_URL"
IMG_MD5_URL="$UBOOT_MTD0_BASE_MD5.$PLATFORM.mtd0.kwb.md5"

FLASH_ERASE_ARGS="$MTD_PART 0 4" # Erase the first 4 blocks, or 512k
NAND_WRITE_ARGS="$MTD_PART $TOOLS/$WHAT_WE_FLASH"
IMG_SIZE="0x80000"
NANDDUMP_ARGS="-f $TOOLS/$WHAT_WE_FLASH.dump $MTD_PART"
ERROR_CALL="error_dont_reboot"

#calling child function to do the work
flash_nand_image

#if the above function does not throw errors and quit the script, then all went well
echo ""
echo "# uBoot installation has completed successfully."

}

flash_envs(){
#checkings to see if there are any modern uboot envs at all, if there are no envs we flash them without asking.
run_fw_printenv_setenv_in_chroot "fw_printenv" ""
if [ $? != "0"  ] ; then
  FLASH_ENVS="Y"
else
  FLASH_ENVS="N"
fi

#checking for the "flash uboot envs anyway" manual override given from command line
if [ "$UPDATE_ENVS" = "YES" ]; then
FLASH_ENVS="Y"
fi

#now we finally check what we need to do and if needed we flash uboot envs
if [ "$FLASH_ENVS" = "Y" ]; then

  WHAT_WE_FLASH="uboot envs"
  MTD_PART="$STOCK_MTD0"
  IMG_URL="$UBOOT_ENVIRONMENT_IMG_URL"
  IMG_MD5_URL="$UBOOT_ENVIRONMENT_IMG_MD5"

  FLASH_ERASE_ARGS="$MTD_PART 0xc0000 1"
  NAND_WRITE_ARGS="-s 786432 $MTD_PART $TOOLS/$WHAT_WE_FLASH"
  IMG_SIZE="0x20000"
  NANDDUMP_ARGS="-s 0xc0000 -f $TOOLS/$WHAT_WE_FLASH.dump $MTD_PART"
  #this is a "error_harmless" because we flash uboot envs BEFORE uboot, so if this fails no damage is done
  ERROR_CALL="error_harmless"

  #calling child function to do the work
  flash_nand_image

fi

#if the above function does not throw errors and quit the script, then all went well
if [ "$FLASH_ENVS" = "Y" ]; then
echo ""
echo "# uBoot environment installation has completed successfully."
else
echo ""
echo "# uBoot environment installation skipped."
fi
}

flash_recovery(){

# Install recovery on NAND

#generating variables we will use to download the right uimage
RECOVERY_UIMAGE="uImage-kirkwood-$RECOVERY_PLATFORM"
RECOVERY_UIMAGE_URL="$MAIN_RECOVERY_URL/$RECOVERY_UIMAGE"
RECOVERY_UIMAGE_MD5_URL="$MAIN_RECOVERY_MD5_URL/$RECOVERY_UIMAGE.md5"


#setting variables for nand flasher function to flash uimage
WHAT_WE_FLASH="recovery-uImage"
MTD_PART="$RECOVERY_KERNEL_MTD"
IMG_URL="$RECOVERY_UIMAGE_URL"
IMG_MD5_URL="$RECOVERY_UIMAGE_MD5_URL"

FLASH_ERASE_ARGS="$MTD_PART 0 0"
NAND_WRITE_ARGS="-p $MTD_PART $TOOLS/$WHAT_WE_FLASH"
#the size will be figured out automatically when the file is downloaded
IMG_SIZE="GET_SIZE"  #"0x1CA8A4" #"1878180" #'$IMG_SIZE'
 # SIZE_IN_HEX="$(printf "%x\n" $IMG_SIZE)"
 # IMG_SIZE="-l 0x$SIZE_IN_HEX"
NANDDUMP_ARGS="-f $TOOLS/$WHAT_WE_FLASH.dump $MTD_PART"
#this isn't very harmless actually, but I plan to have this after uboot install, so if it fails
#debian is still bootable
ERROR_CALL="error_harmless"

#flashing uImage
flash_nand_image

#if the above function does not throw errors and quit the script, then all went well
echo ""
echo "# Recovery uImage installation has completed successfully."

#setting variables for nand flasher function to flash uimage
WHAT_WE_FLASH="recovery-rootfs"
MTD_PART="$RECOVERY_ROOTFS_MTD"
IMG_URL="$RECOVERY_ROOTFS_URL"
IMG_MD5_URL="$RECOVERY_ROOTFS_MD5_URL"

FLASH_ERASE_ARGS="$MTD_PART 0 0"
NAND_WRITE_ARGS="-p $MTD_PART $TOOLS/$WHAT_WE_FLASH"
#the size will be figured out automatically when the file is downloaded
IMG_SIZE="GET_SIZE"
NANDDUMP_ARGS="-f $TOOLS/$WHAT_WE_FLASH.dump $MTD_PART"
#this isn't very harmless actually, but I plan to have this after uboot install, so if it fails
#debian is still bootable
ERROR_CALL="error_harmless"

#flashing rootfs
flash_nand_image

#if the above function does not throw errors and quit the script, then all went well
echo ""
echo "# Recovery rootfs installation has completed successfully."
}

#this still needs to be hooked into error system
gather_telemetry(){
#dumping a bunch of data about the stock firmware and box

#if the platform isn't set we ask what device we are working on
if [ "$PLATFORM" = "" ]; then
echo "please state what device we are analyzing"
read -r PLATFORM
fi

TELEMETRY_FOLDER="$TOOLS/telemetry"
mkdir -p "$TELEMETRY_FOLDER"

#gathering info about stock firmware
TELEMETRY_TEXT="$TELEMETRY_FOLDER/telemetry-$PLATFORM.txt"

for f in "ls -lR" "cat /proc/cpuinfo" "cat /proc/meminfo" "cat /proc/version" "cat /proc/mtd" "cat /proc/cmdline" "cat /proc/swaps" "cat /proc/mdstat" "cat /proc/filesystems" "cat /proc/modules" "df" "mount" "/bin/busybox" "/lib/libc.so.6" "dmesg" "free" "ifconfig" "ps aux" "ps axjf" "blparam" "fw_printenv" ; do

  echo "" >> "$TELEMETRY_TEXT"
  echo "-----------------------------------" >> "$TELEMETRY_TEXT"
  echo "output of $f" >> "$TELEMETRY_TEXT"
  $f 2> /dev/null >> "$TELEMETRY_TEXT"

done

#gathering info using firmware's own tools
  export PATH="$ORIGINAL_PATH"

for f in "blparam" "fw_printenv" ; do

  echo "" >> "$TELEMETRY_TEXT"
  echo "-----------------------------------" >> "$TELEMETRY_TEXT"
  echo "output of stock firmware $f" >> "$TELEMETRY_TEXT"
  $f 2> /dev/null >> "$TELEMETRY_TEXT"

done

#ok done we return to use our own tools
export PATH="$TOOLS"

#dumping the mtd parts that appear to be carrying uboot or envs
#giving the dump the same name it had in stock mtdparts
for f in "1" "2" "3" "4" "5" "6" "7" "8" ; do

  MTD_TO_DUMP=$(cat "/proc/mtd" | grep boot | sed -n "$f"p | awk '{print $1}' | sed 's/.$//' )
  MTD_NAME=$(cat "/proc/mtd" | grep boot | sed -n "$f"p | awk '{print $4}' |  sed 's/"//g' )
  if [ "$MTD_TO_DUMP" != "" ]; then
    nanddump --noecc --omitoob -f "$TELEMETRY_FOLDER/$MTD_NAME-$PLATFORM-$MTD_TO_DUMP" "/dev/$MTD_TO_DUMP"
  fi

done

#setting the date
DATE="$(date +%F_%R)"

#setting the archive name
ARCHIVE_NAME="$PLATFORM-telemetry-$DATE.tar"

#bundling stuff, ready for uploading
cd $TOOLS || return 1
tar -cvf "$ARCHIVE_NAME" "$TELEMETRY_FOLDER"
bzip2 "$ARCHIVE_NAME"

#uploading to ftp server
#ftpput -v -u "anonymous" -p "" "alby.ftp.sh" "/uploads/$ARCHIVE_NAME.bz2" "$ARCHIVE_NAME.bz2"

cd || return 1

#ftp does not seem to work reliably, so we offer to save also locally the file.
#if the disk to work with isn't defined, we run the function to decide what drive we work with
  if [ "$DISK" = "" ]; then
    decide_what_drive_we_work_with
  fi
echo
echo "---unmounting installation drive"
echo

umount -l "$PART_1"
umount -l "$PART_2"


###
#  make dirs & mount
echo "---mounting partitions"
mkdir -p "/tmp/rootfs"

echo
echo "partition to mount is $PART_1"
sleep 5

mount "$PART_1" "/tmp/rootfs"

echo "mounted partitions are"
mount
echo
echo

echo "---checking that the stuff was mounted and writable"
# Just because it's better to.
echo "testing if this is readable" > "/tmp/rootfs/testfile"
if [ ! -f "/tmp/rootfs/testfile" ]; then
    echo "Rootfs partition not created! Aborting!"
    exit 1
fi

#removing the test file
rm "/tmp/rootfs/testfile"
echo "---everything writable"
echo

#moving the telemetry to disk
mv "$ARCHIVE_NAME.bz2" "/tmp/rootfs$ARCHIVE_NAME.bz2"

cd || return 1

}

partition_and_format_disk(){

#if the disk to work with isn't defined, we run the function to decide what drive we work with
if [ "$DISK" = "" ]; then
  decide_what_drive_we_work_with
fi

echo
echo "---unmounting installation drive"
echo

umount -l "$PART_1"
umount -l "$PART_2"

echo
echo "---partitioning installation drive"
echo

#checking install drive size in bytes
install_drive_size_bytes=$(fdisk -l | grep "$DISK" | grep bytes | awk '{print $5}')

#WAAAAAH! the busybox's test cannot deal with so large numbers as it is complited with 32bit integers.
#therefore I'll have to cut the number to shrink it.
install_drive_size_bytes=$(echo $install_drive_size_bytes | sed 's/.\{6\}$//' )

echo "install_drive_size_bytes is $install_drive_size_bytes"

#if storage device is smaller than 1 GB the variable should remain empty, 1073741824 is the actual full number I had to shrink below
if [ "$install_drive_size_bytes" -lt 1073 ]; then
  ERROR="## The rootfs drive size is less than 1 GB."
  error_harmless
fi

#doing a bunch of checks to decide what partition layout to create, 35433480192 is the actual full number I had to shrink below

if [ "$install_drive_size_bytes" -lt 35433 ]; then

  echo "drive is smaller than 33GB so we assume this is a flash drive."
  echo "we will create only a system partition, no data partition."

  fdisk "$DISK" <<EOF
o
n
p
1


w
EOF
  if [ "$?" != "0" ]; then
    ERROR="## Formatting of rootfs failed, installation cannot proceed."
    error_harmless
  fi

  else

  echo "drive is bigger than 33 GB."
  echo "making a 6 GB system partition and"
  echo "the rest of the space becomes a data partition"

  NEED_DATA="yes"

  fdisk "$DISK" <<EOF
o
n
p
1

+7168M
n
p
2


w
EOF
  if [ "$?" != "0" ]; then
    ERROR="## Partitioning of rootfs failed, installation cannot proceed."
    SECOND_MESSAGE="## drive is bigger than 33 GB."
    error_harmless
  fi


fi

fdisk -l

sleep 2
echo "---pausing for a bit so ioctl can re-sync partitions"
sleep 10

#  make filesystems
echo "---formatting the partitions we created"

#we need to unmount aggressively as the stock firmware mounts stuff aggressively
umount -l "$PART_1"
umount -l "$PART_2"

if [ -f "/sbin/e2label" ]; then
# this very likely means we are inside a Linux rootfs,
# labelling it now as rootfs will screw up mounting later
# it will be labelled later when unmounted
mke2fs -t "$FILESYSTEM" "$PART_1"
else
#not in Debian rootfs, so we can give it a label only this way
mke2fs -t "$FILESYSTEM" -L "rootfs" "$PART_1"
fi
if [ "$?" != "0" ]; then
    ERROR="## Formatting of rootfs failed, installation cannot proceed."
    error_harmless
fi

#checking if swap and data partition is needed
if [ "$NEED_DATA" = "yes" ]; then
mke2fs -t "ext4" -L "data" "$PART_2"
fi

umount -l "$PART_1"
umount -l "$PART_2"


echo '---doing file system check on partitons to be sure that they are ok'
e2fsck -fp "$PART_1"
if [ "$?" != "0" ]; then
    ERROR="## Checking of rootfs failed, installation cannot proceed."
    error_harmless
fi
if [ "$NEED_DATA" = "yes" ]; then
e2fsck -fp "$PART_2"
fi
}

preserve_crucial_envs_for_later(){

#preserving crucial envs so we can add them again later

  # Preserve the MAC address

  #read from stock uboot
  read_env_stock_uboot "ethaddr" "no-error"
  if [ $VALUE = "" ]; then
    echo "failed read from stock uboot"
    #read from modern uboot
    read_env_modern_uboot "ethaddr"
    if [ $VALUE = "" ]; then
      echo "it still fails, we try to get the info to fill this variable by ifconfig eth0"
      ENV_ETHADDR=$(ifconfig -a | grep eth0 | awk '{ print $5 }')
      if [ "$ENV_ETHADDR" = "" ] ; then
        echo "it still fails, we try to get the info to fill this variable by ifconfig egiga0"
        ENV_ETHADDR=$(ifconfig -a | grep egiga0 | awk '{ print $5 }')
        if [ "$ENV_ETHADDR" = "" ] ; then
          #still empty, aborting
          ERROR="## Could not read ethaddr uboot variable."
          error_harmless
        fi
      fi
    fi
  fi

  echo "env ethaddr is $ENV_ETHADDR"

    #if it is sill empty then the VALUE is full, so we load it
    if [ "$ENV_ETHADDR" = "" ] ; then
      ENV_ETHADDR="$VALUE"
    fi

  echo "env ethaddr is $ENV_ETHADDR"

  # Preserve the MAC address of the second ethernet port (if it's there)
  #read from stock uboot
  read_env_stock_uboot "eth1addr" "no-error"
  if [ $VALUE = "" ]; then
     echo "failed read from stock uboot"
    #read from modern uboot
    read_env_modern_uboot "eth1addr"
    if [ $VALUE = "" ]; then
      echo "it still fails we try to get the info to fill this variable by ifconfig eth1"
      ENV_ETH1ADDR=$(ifconfig -a | grep eth1 | awk '{ print $5 }')
      if [ "$ENV_ETH1ADDR" = "" ] ; then
        echo "it still fails, we try to get the info to fill this variable by ifconfig egiga1"
        ENV_ETHADDR=$(ifconfig -a | grep egiga1 | awk '{ print $5 }')
        if [ "$ENV_ETHADDR" = "" ] ; then
          #still empty, since having two eth is uncommon in kirkwood boxes,
          #I assume this isn't an error and the box really does not have eth1.
          #so I clean the VALUE so we are sure the variable is set as empty later.
          VALUE=""
        fi
      fi
    fi
  fi
  echo "env eth1addr is $ENV_ETH1ADDR"

    #if it is sill empty then the VALUE is full, so we load it
    if [ "$ENV_ETH1ADDR" = "" ] ; then
      ENV_ETH1ADDR="$VALUE"
    fi

  echo "env eth1addr is $ENV_ETH1ADDR"


  # Preserve the mtdparts
  read_env_stock_uboot "mtdparts" "no-error"
  echo "read mtdparts as $VALUE"
  if [ "$VALUE" = "" ]; then
    #if the above command failed, we proceed to acquire this info in other ways still from stock uboot

    #usually the mtdparts is in the bootargs variable, so we try that
    read_env_stock_uboot "bootargs"
    echo "read bootargs as $VALUE"

    #now we extract the mtdparts from that line
    #and we also drop the "mtdparts=nand_mtd:" part as we need that to be "mtdparts=orion_nand:"
    MTDPARTS=$( printf "%s" "$VALUE" | awk '{ print $1, "\n" $2, "\n" $3, "\n" $4, "\n" $5, "\n" }' | grep "mtdparts" | awk -F  ":" '{print $2}')

    echo "MTDPARTS is $MTDPARTS"

    #if it is empty the above has failed so we try another thing
    if [ "$MTDPARTS" = "" ] ; then
    # we read it from modern uboot
      read_env_modern_uboot "mtdparts"
      if [ $VALUE = "" ]; then
        #still failed or empty, aborting
        ERROR="## Could not read mtdparts uboot variable."
        error_harmless
      fi
      echo "MTDPARTS is $MTDPARTS"
      #still removing the "mtdparts=nand_mtd:" part
      MTDPARTS=$(printf "%s" "$VALUE" | grep "mtdparts" | awk -F  ":" '{print $2}')
    fi
  fi

  #if the MTDPARTS is still empty it means the value in $VALUE is correct and we skipped the above checks,
  #therefore we load it still dropping the part we don't want
  if [ "$MTDPARTS" = "" ] ; then
  MTDPARTS=$(printf "%s" "$VALUE" | grep "mtdparts" | awk -F  ":" '{print $2}')
  fi

  #cool beans, if we got this far we should have a MTDPARTS loaded
  #therefore we add the "mtdparts=orion_nand:" and generate the actual env value

  ENV_MTDPARTS="mtdparts=orion_nand:$MTDPARTS"

  echo "ENV_MTDPARTS is $ENV_MTDPARTS"

}

remove_uboot_tools(){

rm "$BLPARAM"
rm "$NANDWRITE"
rm "$NANDDUMP"
rm "$FLASH_ERASE"
rm "$FW_PRINTENV"
rm "$FW_SETENV"
rm -r "$FW_CONFIG"

}

remove_formatting_tools(){

rm "$E2FSCK"
rm "$MKE2FS"

}

remove_all_tools(){

rm -rf "$TOOLS/*"

}

set_crucial_envs(){
#writing all envs that MUST be set for the box to boot/work at all

#running all commands inside a chroot so we call the same function making/removing the chroot

# writing MAC back
run_fw_printenv_setenv_in_chroot "fw_setenv" "ethaddr $ENV_ETHADDR"
if [ "$?" != "0" ]; then
    ERROR="## Could not set ethaddr uboot env."
    error_harmless
  fi

if [ "$ENV_ETH1ADDR" != "" ] ; then
      # writing MAC of second port back
      run_fw_printenv_setenv_in_chroot "fw_setenv" "eth1addr $ENV_ETH1ADDR"
      if [ "$?" != "0" ]; then
        ERROR="## Could not set eth1addr uboot env."
        error_harmless
      fi
fi

# writing mtdparts back
run_fw_printenv_setenv_in_chroot "fw_setenv" "mtdparts $ENV_MTDPARTS"
if [ "$?" != "0" ]; then
    ERROR="## Could not set mtdparts uboot env."
    error_harmless
fi

# adding stuff that was not there in stock uboot but we need
# writing dtb
run_fw_printenv_setenv_in_chroot "fw_setenv" "dtb_file $ENV_DTB_FILE"
if [ "$?" != "0" ]; then
    ERROR="## Could not set dtb_file uboot env."
    error_harmless
fi

# writing a workaround to make sure boot from Sata works fine
# This will help the "ide reset" to work properly.
# There seems to be a bug in u-boot that if you do "ide reset" too quickly in succession,
# the SATA drive might have problem spinning up
BOOTCMD_UENV_FIX="run uenv_load; if test $uenv_loaded -eq 1; then run uenv_import; fi; sleep 3"

run_fw_printenv_setenv_in_chroot "fw_setenv" "bootcmd_uenv $BOOTCMD_UENV_FIX"
if [ "$?" != "0" ]; then
    ERROR="## Could not set bootcmd_uenv_sata_fix uboot env."
    error_harmless
fi

# writing arcNumber and machid back, if any, otherwise this same code will clear the variables with same name already there
run_fw_printenv_setenv_in_chroot "fw_setenv" "arcNumber $ARCNUMBER"
if [ "$?" != "0" ]; then
    ERROR="## Could not set arcNumber uboot env."
    error_harmless
fi
run_fw_printenv_setenv_in_chroot "fw_setenv" "machid $MACHID"
if [ "$?" != "0" ]; then
    ERROR="## Could not set machid uboot env."
    error_harmless
fi
echo
echo "now the full envs of new uboot are:"
echo

run_fw_printenv_setenv_in_chroot "fw_printenv"


#writing custom boot params (mostly systemd)
if [ "$CUSTOM_PARAM" != "" ]; then
  run_fw_printenv_setenv_in_chroot "fw_setenv" "custom_params $CUSTOM_PARAM"
  if [ "$?" != "0" ]; then
    ERROR="## Could not set custom_params uboot env."
    error_harmless
  fi
fi
}

set_uboot_envs_recovery(){

#now we set uboot envs for recovery

run_fw_printenv_setenv_in_chroot "fw_setenv mtdparts_recovery $RECOVERY_MTDPARTS"
if [ "$?" != "0" ]; then
    ERROR="## Could not set mtdparts_recovery uboot env."
    error_harmless
fi

run_fw_printenv_setenv_in_chroot "fw_setenv set_recovery_bootargs setenv bootargs console=ttyS0,115200 rootdelay=10"  '$mtdparts_recovery'
if [ "$?" != "0" ]; then
    ERROR="## Could not set set_recovery_bootargs uboot env."
    error_harmless
fi


#automatically figuring out what nand part we must write in the env
#For example nand0,7 is /dev/mtd7 in the first (only) NAND device
NAND_PARTITION="nand0,$(echo "$RECOVERY_KERNEL_MTD" | sed 's/[^0-9]*//g')"

run_fw_printenv_setenv_in_chroot "fw_setenv bootcmd_recovery run set_recovery_bootargs; nand read 0x800000 $NAND_PARTITION ; bootm 0x800000"
if [ "$?" != "0" ]; then
    ERROR="## Could not set bootcmd_recovery uboot env."
    error_harmless
fi

run_fw_printenv_setenv_in_chroot "fw_setenv bootcmd" "run bootcmd_uenv; run scan_disk; run set_bootargs; run bootcmd_exec; run bootcmd_recovery;"
if [ "$?" != "0" ]; then
    ERROR="## Could not set bootcmd uboot env."
    error_dont_reboot
fi

echo
echo "now the full envs of new uboot with recovery are:"
echo

run_fw_printenv_setenv_in_chroot "fw_printenv"

}


###############################################
#####
##### device or vendor-specific functions
#####
###############################################

zyxel_kill_programs(){

#this must be used for all zyxel devices as the firmware is pretty much the same for all

## killing the zyxel drive automounter daemon that %&£%"£ up everything any time it detects a new partition
#something else is still active and mounts stuff at boot or on hotplug but it does not annoy us otherwise
#killing also other stuff that might be annoying.
killall zyshd
killall myhotplug
killall zyfw_downloader
}

pogoplug_kill_programs(){

#this must be used for all pogoplug E02, pogoplug v4/mobile dockstar, sheevaplug devices as the firmware is pretty much the same for all

killall hbwd
}

###############################################
#####
##### functions that run the whole install process
##### they basically contain a list of macrofunctions above
##### this is to keep tidy the case -- esac below
#####
###############################################


back_up(){
download_tools
check_bad_blocks
download_uboot_tools
backing_up_uboot_and_envs
remove_uboot_tools
remove_all_tools
export PATH="$ORIGINAL_PATH"
echo
echo "###"
echo "###Backup completed, have a good day."
echo "###"
echo
}

install_rootfs(){
download_tools
download_formatting_tools
partition_and_format_disk
download_and_write_rootfs
remove_formatting_tools
remove_all_tools
export PATH="$ORIGINAL_PATH"
echo
echo "###"
echo "###Debian installation completed, have a good day."
echo "###"
echo
}

install_recovery(){
download_tools
download_uboot_tools
flash_recovery
set_uboot_envs_recovery
remove_uboot_tools
remove_all_tools
export PATH="$ORIGINAL_PATH"
echo
echo "###"
echo "###Recovery installation completed, have a good day."
echo "###"
echo
}

install_recovery_and_rootfs(){
download_tools

download_formatting_tools
partition_and_format_disk
download_and_write_rootfs
remove_formatting_tools

download_uboot_tools
flash_recovery
set_uboot_envs_recovery
remove_uboot_tools
remove_all_tools
export PATH="$ORIGINAL_PATH"
echo
echo "###"
echo "###Recovery installation completed, have a good day."
echo "###"
echo
}

install_uboot(){
download_tools
check_bad_blocks
download_uboot_tools
preserve_crucial_envs_for_later
flash_envs
set_crucial_envs

#install uboot MUST be kept last, so that errors in partitioning or writing the rootfs or envs don't require recovery
flash_uboot
remove_uboot_tools
remove_all_tools
export PATH="$ORIGINAL_PATH"
echo
echo "###"
echo "###Uboot installation completed, have a good day."
echo "###"
echo
}

install_uboot_and_recovery(){
download_tools
check_bad_blocks
download_uboot_tools
preserve_crucial_envs_for_later
flash_envs
set_crucial_envs

set_uboot_envs_recovery

#install uboot MUST be kept last, so that errors in partitioning or writing the rootfs or envs don't require recovery
flash_uboot

#flashing recovery should happen after uboot as if it fails we can still boot Debian.
flash_recovery

remove_uboot_tools
remove_all_tools
export PATH="$ORIGINAL_PATH"
echo
echo "###"
echo "###Uboot and recovery installation completed, have a good day."
echo "###"
echo
}

install_uboot_and_rootfs(){
download_tools
check_bad_blocks

download_formatting_tools
partition_and_format_disk
download_and_write_rootfs
remove_formatting_tools

download_uboot_tools
backing_up_uboot_and_envs
preserve_crucial_envs_for_later
flash_envs
set_crucial_envs

#install uboot MUST be kept last, so that errors in partitioning or writing the rootfs or envs don't require recovery
flash_uboot

#flashing recovery should happen after uboot as if it fails we can still boot Debian.
flash_recovery

remove_uboot_tools
remove_all_tools

export PATH="$ORIGINAL_PATH"
end_installation_message_reboot
}

install_all(){
download_tools
check_bad_blocks

download_formatting_tools
partition_and_format_disk
download_and_write_rootfs
remove_formatting_tools

download_uboot_tools
backing_up_uboot_and_envs
preserve_crucial_envs_for_later
flash_envs
set_crucial_envs
set_uboot_envs_recovery

#install uboot MUST be kept last, so that errors in partitioning or writing the rootfs or envs don't require recovery
flash_uboot

#flashing recovery should happen after uboot as if it fails we can still boot Debian.
flash_recovery

remove_uboot_tools
remove_all_tools

export PATH="$ORIGINAL_PATH"
end_installation_message_reboot
}

decide_install_type(){

#depending on what is chosen from command line we decide what install we do
#default setting is full install

case $INSTALL_TYPE in

full)
  install_all
;;

uboot)
  install_uboot
;;

rootfs)
  install_rootfs
;;

recovery)
  install_recovery
;;

uboot-rootfs)
  install_uboot_and_rootfs
;;

uboot-recovery)
  install_uboot_and_recovery
;;

rootfs-recovery)
  install_recovery_and_rootfs
;;

backup)
  back_up
;;

esac
}

###############################################
#####
##### DISABLED FUNCTIONS, dead code that might be useful in future
#####
###############################################

disabled(){
true
# disabled_uboot_check () {
#
#
# # Dump existing uBoot and compare it to a database of known versions
# ###########
# # we don't have stock uboots of most devices, so disabling this for now
# ##########
#
# echo ""
# echo "# Validating existing uBoot..."
#
# # dump the first 512k of mtd0 to /tmp
# $NANDDUMP --noecc --omitoob -l 0x80000 -f /tmp/uboot-mtd0-dump /dev/mtd0
#
# wget -O "/tmp/valid-uboot.md5" "$VALID_UBOOT_MD5"
#
# UPDATE_UBOOT=1
# UBOOT_PLATFORM=
# CURRENT_UBOOT_MD5=$(md5sum "/tmp/uboot-mtd0-dump" | cut -d' ' -f1)
# UBOOT_DETAILS=$(grep "$CURRENT_UBOOT_MD5" /tmp/valid-uboot.md5 | head -1)
# if [ "$UBOOT_DETAILS" != "" ]; then
#   UBOOT_PLATFORM=$(echo "$UBOOT_DETAILS" | sed 's/^\w* \(\w*\) .*$/\1/')
#   UBOOT_VERSION=$(echo "$UBOOT_DETAILS" | sed 's/^\w* \w* \(.*\)$/\1/')
#   echo "## Valid uBoot detected: [$UBOOT_PLATFORM $UBOOT_VERSION]"
# else
#   echo "## Unknown uBoot detected on mtd0: $CURRENT_UBOOT_MD5"
#   echo "##"
#   if [ "$NO_UBOOT_CHECK" != "1" ]; then
#     echo "## The installer could not detect the version of your current uBoot"
#     echo "## This may happen if you have installed a different uBoot on"
#     echo "## /dev/mtd0 or if you have bad blocks on /dev/mtd0"
#     echo "##"
#     echo "## If you have bad blocks on mtd0, you should not try to install uBoot."
# #    echo "##"
# #    echo "## If you have installed a diffirent uBoot on mtd0, and understand the"
# #    echo "## risks, you can re-run the installer with the --no-uboot-check parameter"
#     echo "##"
#     echo "## Installation cancelled."
#     rm "/tmp/valid-uboot.md5"
#     exit 1
#   else
#     echo "## --no-uboot-check flag detected, continuing installation"
#
#     while [ "$UBOOT_PLATFORM" = "" ]; do
#       echo ""
#       echo "############################################"
#       echo "Your device could not be auto-detected."
#       echo ""
#       echo "You must be using a device listed below to run this installer."
#       echo ""
#       echo "What device are you using? Type the number of your device and press ENTER."
#       echo "1 - Seagate Dockstar"
#       echo "2 - Seagate GoFlex Net"
#       echo "3 - Seagate GoFlex Home"
#       echo "4 - Pogoplug v1"
#       echo "5 - Pogoplug v2 - Pink"
#       echo "6 - Other"
#       read -r device
#
#       if [ "$device" = "1" ]; then
#         echo "Selected Dockstar"
#         UBOOT_PLATFORM="dockstar"
#         UBOOT_VERSION="unknown"
#       elif [ "$device" = "2" ]; then
#         echo "Selected Seagate GoFlex Net"
#         UBOOT_PLATFORM="goflexnet"
#         UBOOT_VERSION="unknown"
#       elif [ "$device" = "3" ]; then
#         echo "Selected Seagate GoFlex Home"
#         UBOOT_PLATFORM="goflexhome"
#         UBOOT_VERSION="unknown"
#       elif [ "$device" = "4" ]; then
#         echo "Selected Pogoplug v1"
#         UBOOT_PLATFORM="pogov1"
#         UBOOT_VERSION="unknown"
#       elif [ "$device" = "5" ]; then
#         echo "Selected Pogoplug v2 - Pink"
#         UBOOT_PLATFORM="pinkpogo"
#         UBOOT_VERSION="unknown"
#       elif [ "$device" = "6" ]; then
#         echo "Selected Other Device, exiting"
#         echo "This installer is only compatible with the listed devices."
#         exit 1
#       else
#         echo "Invalid Input"
#       fi
#     done
#
#   fi
# fi
#
# if [ "$FORCE_PLATFORM" != "" ] && [ "$FORCE_PLATFORM" != "$UBOOT_PLATFORM" ]; then
#   echo "## --force-platform paramater [$FORCE_PLATFORM] does not match detected platform [$UBOOT_PLATFORM]."
#   echo "Are you sure your device is a $FORCE_PLATFORM? [y/N] "
#   read -r FORCE
#   if [ "$FORCE" = "y" ] || [ "$FORCE" = "Y" ]; then
#     echo "## Forcing installation of [$FORCE_PLATFORM] platform"
#     UBOOT_PLATFORM=$FORCE_PLATFORM
#     UBOOT_VERSION="unknown"
#   else
#     echo "## Exiting. No changes were made to mtd0."
#     exit 1
#   fi
# fi
#
# UBOOT_IS_CURRENT=$(echo $UBOOT_VERSION | grep -c current)
# if [ "$UBOOT_IS_CURRENT" = "1" ]; then
#   echo "## The newest uBoot is already installed on mtd0."
#   UPDATE_UBOOT=0
# else
#   UBOOT_CURRENT=$(grep "$UBOOT_PLATFORM" /tmp/valid-uboot.md5 | grep current | head -1 | sed 's/^\w* \w* \(.*\)-current$/\1/')
# fi
#
# rm "/tmp/valid-uboot.md5"
#
# }
#
# disabled_service_disabler() {
#
# # If this is the first time this installer has been run in the
# # original Pogoplug enviroment, check if the user wants to disable
# # the Pogoplug services
# #########
# #this stuff will likely be split into device-specific functions,
# #for now it stays disabled
# #########
#
# if [ -d /usr/local/cloudengines/ ] && [ ! -e "$UBOOT_ORIGINAL" ]; then
#   killall hbwd
#   echo ""
#   echo ""
#   echo ""
#   echo "DISABLE POGOPLUG SERVICES"
#   echo ""
#   echo "The pogoplug service includes an auto-update feature which could"
#   echo "be used to cripple or disable your device.  It is recommended"
#   echo "that you disable this service."
#   echo ""
#   echo "NOTE: The pogoplug service is proprietary software"
#   echo "created by Cloud Engines.  It is not available for use"
#   echo "in other distributions and will not be available in"
#   echo "your new linux installation even if you choose not to disable it."
#   echo ""
#   echo "Would you like to disable the pogoplug services? [Y/n] "
#   read -r DISABLE
#   if [ "$DISABLE" = "" ] || [ "$DISABLE" = "y" ] || [ "$DISABLE" = "Y" ];
#   then
#
#     echo "Applying fixes to the pogoplug environment..."
#
#     mount -o rw,remount /
#
#     # Add /sbin to the path and cleanup the shell prompt
#     if [ ! -f "/root/.bash_profile" ]; then
#       echo -e \
#   "export PS1='\h:\w\$ '
#   export PATH='/usr/bin:/bin:/sbin'
#   " > /root/.bash_profile
#     fi
#
#     chmod go+w /dev/null
#
#     # Re-enable dropbear (updated dockstars only)
#     sed -i 's/^#\/usr\/sbin\/dropbear/\/usr\/sbin\/dropbear/' /etc/init.d/db > /dev/null 2>&1
#
#     echo "Disabling the pogoplug service..."
#     # Comment out the line that starts hmbgr.sh
#     sed -i 's/^\/etc\/init.d\/hbmgr.sh start/#Uncomment the line below to enable the pogoplug service\n#\/etc\/init.d\/hbmgr.sh start/' /etc/init.d/rcS
#
#     mount -o ro,remount /
#
#     echo "Done fixing pogoplug environment."
#     echo ""
#   fi
#
#   UBOOT_ORIGINAL_URL="$UBOOT_MTD0_BASE_URL.$UBOOT_PLATFORM.original.kwb"
#   install "$UBOOT_ORIGINAL"   "$UBOOT_ORIGINAL_URL"    644
#
#   #install "$BLPARAM"          "$BLPARAM_URL"           755 we don't have blparam
#
#   if   [ "$UBOOT_PLATFORM" = "pinkpogo"  ]; then BOOTCMD='nand read.e 0x800000 0x100000 0x200000; setenv bootargs $(console) $(bootargs_root); bootm 0x800000'
#   # dockstar, goflex, and pogov1 have the same bootcmd
#   else BOOTCMD='nand read.e 0x800000 0x100000 0x300000; setenv bootargs $(console) $(bootargs_root); bootm 0x800000'
#   fi
#   #$BLPARAM "bootcmd=$BOOTCMD" > /dev/null 2>&1
#
#   # Preserve the MAC address
#   #ENV_ETHADDR=`$BLPARAM | grep "^ethaddr=" | cut -d'=' -f 2-`
# fi
# }
#
# disabled_preserve_custom_envs_for_later() {
#
# #this is not working, currently not used
#
#   # Preserve the 'rescue_installed' setting
#   ENV_RESCUE_INSTALLED=`$FW_PRINTENV rescue_installed 2> /dev/null | cut -d'=' -f 2-`
#   if [ "$ENV_RESCUE_INSTALLED" = "" ]; then
#     ENV_BOOTCMD_RESCUE=`$FW_PRINTENV bootcmd_rescue 2> /dev/null`
#     if [ "$ENV_BOOTCMD_RESCUE" != "" ]; then
#       ENV_RESCUE_INSTALLED=1
#     fi
#   fi
#
#   # Preserve the arcNumber value
#   ENV_ARCNUMBER=`$FW_PRINTENV arcNumber 2> /dev/null | cut -d'=' -f 2-`
#
#   # Preserve the custom kernel parameters
#   ENV_RESCUE_CUSTOM=`$FW_PRINTENV rescue_custom_params 2> /dev/null | cut -d'=' -f 2-`
#   ENV_USB_CUSTOM=`$FW_PRINTENV usb_custom_params 2> /dev/null | cut -d'=' -f 2-`
#   ENV_UBIFS_CUSTOM=`$FW_PRINTENV ubifs_custom_params 2> /dev/null | cut -d'=' -f 2-`
# }
#
# disabled_reload_custom_envs(){
#
# # not working and not used, writing custom envs again from backup
#
#   $FW_SETENV ethaddr $ENV_ETHADDR
#   if [ "$ENV_RESCUE_INSTALLED" = "1" ]; then $FW_SETENV rescue_installed $ENV_RESCUE_INSTALLED; fi
#   if [ "$ENV_RESCUE_CUSTOM" != "" ]; then $FW_SETENV rescue_custom_params $ENV_RESCUE_CUSTOM; fi
#   if [ "$ENV_USB_CUSTOM" != "" ]; then $FW_SETENV usb_custom_params $ENV_USB_CUSTOM; fi
#   if [ "$ENV_UBIFS_CUSTOM" != "" ]; then $FW_SETENV ubifs_custom_params $ENV_UBIFS_CUSTOM; fi
# }
#
# #this works fine, but until we have a sgdisk we can't use it
# GPT_MBR_partition_and_format_disk() {
#
# #by default the device to use is this
# DISK="/dev/sda"
#
# #checking if there are drives over a GiB at all
# DETECTED_DRIVE=$(sgdisk -p /dev/sda | grep /dev | grep GiB )
#
# #checking if we see a SDcard in a true SDcard slot over a GiB in size
# DETECTED_MMC=$(sgdisk -p /dev/mmcblk0 | grep /dev | grep GiB )
#
# #if both searches turn out empty, we have no drives to work with
# if [[ "$DETECTED_DRIVE" = "" ]] && [[ "$DETECTED_MMC" = "" ]] ; then
#   echo "No disks or SD cards detected. Aborting!"
#   exit 1
# fi
#
# #if no drives are detected but a SDcard is detected, we select that for partitioning
# if [[ "$DETECTED_DRIVE" = "" ]] && [[ "$DETECTED_MMC" != "" ]] ; then
#   DISK="/dev/mmcblk0"
# fi
#
# #checking if there is another drive or if we detected one SDcard and no disks above so we can skip questions
# DETECTED_DRIVE=$(sgdisk -p /dev/sdb | grep /dev | grep GiB  )
# if [[ "$DETECTED_DRIVE" = "" ]] || [[ "$DISK" = "/dev/mmcblk0" ]] ; then
#   #if conditions are true we skip the else block and proceed with /dev/sda
#   DISK="/dev/sda"
# else
#   #ok, there is more than one drive in this box.
#   #since for various reasons we cannot get a RELIABLE way to find what device is friggin /
#   #and in case of multiple drives it makes more sense anyway, we ask the user what drive he wants
#
#   echo
#   echo
#   echo "what drive you want to install Debian in? "
#   echo "(write 1 or 2 or 3 or 4 or sd to select the drive)"
#   echo "(write Q if you want to quit)"
#   echo
#   DISK=$(sgdisk -p /dev/sda | grep /dev | grep GiB  )
#   if [[ "$DISK" == *"Gib"* ]]; then
#     echo "Disk 1 size: $(echo "$DISK" | awk '{ print $5, $6 }')"
#   fi
#
#   DISK=$(sgdisk -p /dev/sdb | grep /dev | grep GiB  )
#   if [[ "$DISK" == *"Gib"* ]]; then
#     echo "Disk 2 size: $(echo "$DISK" | awk '{ print $5, $6 }')"
#   fi
#
#   DISK=$(sgdisk -p /dev/sdc | grep /dev | grep GiB  )
#   if [[ "$DISK" == *"Gib"* ]]; then
#     echo "Disk 3 size: $(echo "$DISK" | awk '{ print $5, $6 }')"
#   fi
#
#   DISK=$(sgdisk -p /dev/sdd | grep /dev | grep GiB  )
#   if [[ "$DISK" == *"Gib"* ]]; then
#     echo "Disk 4 size: $(echo "$DISK" | awk '{ print $5, $6 }')"
#   fi
#
#   DISK=$(sgdisk -p /dev/mmcblk0 | grep /dev | grep GiB  )
#   if [[ "$DISK" == *"Gib"* ]]; then
#     echo "SDcard size: $(echo "$DISK" | awk '{ print $5, $6 }')"
#   fi
#
#   read -r SELECTION
#
#   case $SELECTION in
#
#     1)
#       DISK="/dev/sda"
#     ;;
#     2)
#       DISK="/dev/sdb"
#     ;;
#     3)
#       DISK="/dev/sdc"
#     ;;
#     4)
#       DISK="/dev/sdd"
#     ;;
#     sd)
#       DISK="/dev/mmcblk0"
#     ;;
#   esac
# fi
#
# echo
# echo "installation drive is $DISK"
# echo
#
# PART_1="${DISK}1"
# PART_2="${DISK}2"
#
# echo
# echo "---unmounting installation drive"
# echo
#
# #deactivating raids
# for f in "/dev/md0" "/dev/md1" "/dev/md2" "/dev/md3" "/dev/md4" ; do
#
#   umount -l "$f"
#   export PATH="$ORIGINAL_PATH"
#   mdadm --stop "$f"
#   export PATH="$TOOLS"
# done
#
# #unmounting plain partitions if they were automounted
# umount -l "$PART_1"
# umount -l "$PART_2"
#
# echo
# echo "---partitioning installation drive"
# echo
#
# #checking install drive size
# DRIVE_SIZE=$(sgdisk -p $DISK | grep /dev | awk '{print $5}')
#
# echo "install_drive_size is $DRIVE_SIZE"
#
# # turning the size value into an integer
# DRIVE_SIZE_INTEGER=$( echo "$DRIVE_SIZE" | sed 's/\..*//' )
#
# echo "DRIVE_SIZE_INTEGER is $DRIVE_SIZE_INTEGER"
#
# #doing a bunch of checks to decide what partition layout to create.
#
# if (( "$DRIVE_SIZE_INTEGER" < 33 )); then
#
#   echo "drive is smaller than 33GB so we assume this is a flash drive."
#   echo "we will create only a system partition, no data partition."
#
#   #checking if the user called the script with GPT option
#   if [[  $PARTITION_TABLE = "GPT" ]]; then
#
#     sgdisk -o -n1:64:0 -t1:8300 -g $DISK
#
#   else
#     #else we partition in MBR
#     fdisk "$DISK" <<EOF
# o
# n
# p
# 1
#
#
# w
# EOF
#   fi
# fi
#
# if (( "$DRIVE_SIZE_INTEGER" < 2048 )); then
#
#   echo "drive is bigger than 33GB but smaller than 2TB so we assume this is a hard drive."
#   echo "making a 7 GB system partition, the rest becomes a data partition"
#
#   NEED_DATA="yes"
#
#   #checking if the user called the script with GPT option
#   if [[  $PARTITION_TABLE = "GPT" ]]; then
#
#       sgdisk -o -n1:64:+7G -n2:0:0 -t1:8300 -t2:8300 -g $DISK
#
#     else
#       #else we partition in MBR
#       fdisk "$DISK" <<EOF
# o
# n
# p
# 1
#
# +7168M
# n
# p
# 2
#
#
# w
# EOF
#   fi
#
# fi
#
# if (( "$DRIVE_SIZE_INTEGER" > 2048 )); then
#
#   echo "drive is bigger than 2TB so we must format it as GPT"
#   echo "only the new uboot can boot drives formatted as GPT, original uboot cannot"
#   echo "making a 7 GB system partition, the rest becomes a data partition"
#
#   NEED_DATA="yes"
#
#   sgdisk -o -n1:64:+7G -n2:0:0 -t1:8300 -t2:8300 -g $DISK
#
# fi
#
# echo "---pausing for a bit so ioctl can re-sync partitions"
# sleep 10
#
# #  make filesystems
# echo "---formatting the partitions we created"
#
# if [ -f "/sbin/e2label" ]; then
#   # this means we are inside a Debian rootfs,
#   # labelling it now as rootfs will screw up mounting later
#   # it will be labelled later when unmounted
#   mke2fs -t "$FILESYSTEM" "$PART_1"
#
# else
#   #we are not in a Debian rootfs, so we can give it a label only this way
#   mke2fs -t "$FILESYSTEM" -L "rootfs" "$PART_1"
# fi
#
# #checking if swap and data partition is needed
# if [ "$NEED_DATA" = "yes" ]; then
#   mke2fs -t "ext4" -L "data" "$PART_2"
# fi
#
# echo '---doing file system check on partitons to be sure that they are ok'
#
# e2fsck -fp "$PART_1"
#
# if [ "$NEED_DATA" = "yes" ]; then
#   e2fsck -fp "$PART_2"
# fi
# }
}

################################################
#####
##### MAIN
#####
################################################

#doing a root check, because the installer must be run as root for obvious reasons
if [ "$(id -u)" -eq 0 ]; then
	echo ;
else
	echo "you must run this script as root";
exit
fi


###default settings for script operation
CUSTOM_PARAM=""
DEVICE_TYPE=""
MAKE_ROOTFS="YES"
UPDATE_ENVS=""
FILESYSTEM="ext3"
PARTITION_TABLE="MBR"
INSTALL_TYPE="full"

##loading advanced options passed as arguments if any, and changing relevant variables
for i in "$@" ; do

#this is needed to remove a = and anything after it, so we can select the right argument first
#it allows users to send a -device=nsa310 as commandline argument and the script will select that device
arg="$(echo "$i" | sed 's/=.*//')"

case $arg in
    -GPT) #this is there but it is not used because sgdisk does not work.
      #PARTITION_TABLE="GPT"
    ;;
    -ext4)
      FILESYSTEM="ext4"
    ;;
    -device) # and here we remove the = and anything before it to load the device type
      DEVICE_TYPE="$(echo "$i" | sed 's/.*=//')"
    ;;
    -drive) # and here we remove the = and anything before it to load the drive we want Debian in
      DISK="$(echo "$i" | sed 's/.*=//')"
    ;;
    -systemd)
    CUSTOM_PARAM="$CUSTOM_PARAM init=/bin/systemd"
    ;;
    -update-envs)
      UPDATE_ENVS="YES"
    ;;
    -install-uboot)
      INSTALL_TYPE="uboot"
    ;;
    -install-rootfs)
      INSTALL_TYPE="rootfs"
    ;;
    -install-recovery)
      INSTALL_TYPE="recovery"
    ;;
    -install-uboot-rootfs)
      INSTALL_TYPE="uboot-rootfs"
    ;;
    -install-uboot-recovery)
      INSTALL_TYPE="uboot-recovery"
    ;;
    -install-rootfs-recovery)
    INSTALL_TYPE="rootfs-recovery"
    ;;
    -backup)
      INSTALL_TYPE="backup"
    ;;
    *)
           echo "argument $i not recognized"
           exit 1
    ;;
esac
done

#splitting the text depending on what install is selected
#mostly to provide some feedback as to what was selected
echo ""
echo ""
echo "!!!!!!  WARNING: PLEASE READ THIS CAREFULLY  !!!!!!"
echo ""

if [ "$INSTALL_TYPE" = "full" ] || [ "$INSTALL_TYPE" = "uboot" ] || [ "$INSTALL_TYPE" = "uboot-rootfs" ] || [ "$INSTALL_TYPE" = "uboot-recovery" ] ; then
echo
echo "We will install a new bootloader (u-boot) in the"
echo "internal memory of the device."
echo
echo "Afterwards, booting the stock firmware will not be possible."
echo
echo "The new bootloader can also boot fully-fledged third party "
echo "firmwares like OpenWRT or LEDE but installing them is "
echo "outside the scope of this installer."
fi

if [ "$INSTALL_TYPE" = "full" ] || [ "$INSTALL_TYPE" = "recovery" ] || [ "$INSTALL_TYPE" = "uboot-recovery" ] || [ "$INSTALL_TYPE" = "rootfs-recovery" ] ; then
echo
echo "We will replace the stock firmware with a powerful recovery "
echo "firmware (a customized LEDE/OpenWRT)."
echo "So if your Debian system fails to start up for some reason,"
echo "you can just remove any drive and reboot the device, "
echo "it will start the recovery system."
echo "From there you will be able to do all you can do from a "
echo "modern Debian system, to rescue or create a new rootfs"
fi

if [ "$INSTALL_TYPE" = "full" ] || [ "$INSTALL_TYPE" = "uboot" ] || [ "$INSTALL_TYPE" = "uboot-rootfs" ] || [ "$INSTALL_TYPE" = "uboot-recovery" ] ; then
echo
echo "WARNING:If you lose power while the bootloader is being written,"
echo "your device could be left in an unusable state and require "
echo "serial connection to recover, (ask for help in the forums)."
echo
echo "This dangerous part is done in a few seconds though, "
echo "this script spends most of the time doing other interruptible "
echo "jobs like installing Debian inside a drive."
echo "Therefore, doing permanent damage (bricking your device) is unlikely."
fi

if [ "$INSTALL_TYPE" = "full" ] || [ "$INSTALL_TYPE" = "rootfs" ] || [ "$INSTALL_TYPE" = "uboot-rootfs" ] || [ "$INSTALL_TYPE" = "rootfs-recovery" ] ; then
echo
echo "Please make sure that there is a USB, Sata or SDcard"
echo "so we can install Debian in it."
echo "We need at least 2GB of storage space, the faster it is the better."
echo ""
echo "THIS SCRIPT WILL DESTROY ALL EXISTING DATA ON THE STORAGE DEVICE WE ARE INSTALLING DEBIAN IN."
echo "PLEASE MOVE YOUR DATA AWAY FROM ANY DRIVE CONNECTED TO THE DEVICE OR FACE DATA LOSS"
echo ""
fi

echo "By typing ok, you agree to assume all liabilities and risks associated with running this installer."
echo ""
echo "If everything looks good, type 'ok' to continue: "


read -r IS_OK
if [ "$IS_OK" != "OK" ] && [ "$IS_OK" != "Ok" ] && [ "$IS_OK" != "ok" ]
then
  echo "Exiting..."
  exit
fi

#if we haven't set the device type from command line arguments, we ask the user
if [ "$DEVICE_TYPE" = "" ]; then
  #SELECTING DEVICE
  echo
  echo
  echo "Now we need to know what device we are working on, here is a list, please write the number of the device"
  echo

  #add here all devices supported
  echo "0.For Expert Users"
  echo "1.Zyxel nsa310"
  echo "2.Zyxel nsa325 (v1 and v2)"
  #echo "3.Pogoplug"

  #loading the user selection
  read -r DEVICE_TYPE
fi

# and here goes all device-specific operations
case "$DEVICE_TYPE" in

        X)    echo "running test X"
          download_tools
          download_formatting_tools
          download_uboot_tools
          echo
          echo "we will now read the bootcmd, then try reading a non-existing env"
          read_env_stock_uboot "bootcmd"
          read_env_stock_uboot "NON-EXISTING_ENV"

        ;;

    0)  # For Expert Users
      echo "00. download and unpack toolbox"
      echo "01. make a Debian rootfs in a drive"
      echo "02. gathering info about this device"
      #loading the user selection
      read -r DEVICE_TYPE

      # and here goes all device-specific operations
      case "$DEVICE_TYPE" in

        00)    echo "downloading tools"
          download_tools
          download_formatting_tools
          download_uboot_tools
          echo ""
          echo ""
          echo "tools downloaded in $TOOLS, busybox tools installed"
          echo "you may want to write " "export PATH=$TOOLS:$PATH"
          echo "so you can use them as default tools for this session"
        ;;

        01)    echo "downloading tools and making a rootfs"
          install_rootfs
        ;;

        02) echo "gathering info about this device"
          download_tools
          download_uboot_tools
          gather_telemetry
        ;;

      esac
    ;;

	1) #zyxel nsa310
		PLATFORM="nsa310" # platform name used inside uboot name
		STOCK_MTD0="/dev/mtd0" #this is the how the mtd0 is, as seen from inside the stock firmware
		ENV_DTB_FILE="/boot/dts/kirkwood-nsa310b-lmsensor-redusbled.dtb" # this is the env to add the dtb
		MACHID="" # leave empty if none
		ARCNUMBER="4022" # leave empty if none

		#recovery installation settings
		#used to generate the name of the uimage to download and flash
		RECOVERY_PLATFORM="nsa310b-lmsensor-redusbled"
		#mtd partitions of kernel and rootfs
        RECOVERY_KERNEL_MTD="/dev/mtd7"
        RECOVERY_ROOTFS_MTD="/dev/mtd8"
        #the mtdpart with the kernel must be called "kernel" and the mtdpart with the rootfs must be called "rootfs"
        #otherwise LEDE kernel won't recognize the rootfs and will not boot properly, as their kernel is patched to split any "rootfs" partition in a squashfs and a dynamically resized jffs2 after it.
        #stating the rootfs in bootargs is useless.
		RECOVERY_MTDPARTS="mtdparts=orion_nand:0x100000(uboot),0x80000(uboot_env),0x80000(key_store),0x80000(info),0xA00000(etc),0xA00000(nope_kernel_1),0x2FC0000(nope_rootfs1),0xA00000(kernel),0x2FC0000(rootfs)"

		# the uboot download link variable is written above as uboot links are likely to change
		# more often than most of the data here, that should be write-once-and-never-touch-again
		UBOOT_URL=$UBOOT_NSA310_URL

        #running a killer function to kill off a bunch of stock FW daemons that get in the way
        zyxel_kill_programs

        decide_install_type

	;;
	2) #zyxel nsa325 INCOMPLETE
      echo "disabled"
      exit 1
		PLATFORM="nsa325"
		UBOOT_URL=$UBOOT_NSA325_URL
		STOCK_MTD0="/dev/mtd0" #this is the how the mtd0 is, as seen from inside the stock firmware
		ENV_DTB_FILE="/boot/dts/kirkwood-nsa325.dtb"
		MACHID="" # leave empty if none
		ARCNUMBER="4495" # leave empty if none

		# the uboot download link variable is written above as uboot links are likely to change
		# more often than most of the data here, that should be write-once-and-never-touch-again
		UBOOT_URL=$UBOOT_NSA310_URL

        #running a killer function to disable features that get in the way
        zyxel_kill_programs

		decide_install_type
	;;
	3) #Pogoplug V4/Mobile
		PLATFORM="pogo_v4"
        UBOOT_URL=$UBOOT_POGO_V4_URL
		STOCK_MTD0="/dev/mtd0" #this is the how the mtd0 is, as seen from inside the stock firmware
		ENV_DTB_FILE="/boot/dts/kirkwood-pogoplug_v4.dtb"
        MACHID="F78" # leave empty if none
        ARCNUMBER="3960" # leave empty if none

        #recovery installation settings
		#used to generate the name of the uimage to download and flash
		RECOVERY_PLATFORM="pogoplug_v4"
		#mtd partitions of kernel and rootfs
        RECOVERY_KERNEL_MTD= #fill this
        RECOVERY_ROOTFS_MTD= #fill this

        #the mtdpart with the kernel must be called "kernel" and the mtdpart with the rootfs must be called "rootfs"
        #otherwise LEDE kernel won't recognize the rootfs and will not boot properly, as their kernel is patched to split any "rootfs" partition in a squashfs and a dynamically resized jffs2 after it.
        #stating the rootfs in bootargs is useless.
		RECOVERY_MTDPARTS= #fill this



        # the uboot download link variable is written above as uboot links are likely to change
        # more often than most of the data here, that should be write-once-and-never-touch-again
        UBOOT_URL=$UBOOT_POGO_V4_URL

		#device-specific commands to be run before proceding
        pogoplug_kill_programs

		decide_install_type

	;;

esac

#getting the path back to default
export PATH="$ORIGINAL_PATH"

exit 0
